-------------------------------------------------------------------------------
--=> weizPVP.lua
-- weizPVP CORE
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> DEBUG GLOBALS
-- luacheck: globals ViragDevTool_AddData
--> weizPVP Globals
-- luacheck: globals weizPVP
--> Libs
-- luacheck: globals LibStub
--> WoW Globals
-- luacheck: globals table.wipe GetRealmName GetUnitName UnitLevel
-------------------------------------------------------------------------------
--=> LIBS
-------------------------------------------------------------------------------
weizPVP = LibStub("AceAddon-3.0"):NewAddon("weizPVP", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
--=> GLOBALS TO LOCALS
-------------------------------------------------------------------------------
local table_wipe = table.wipe

-------------------------------------------------------------------------------
--=> GLOBALS
-------------------------------------------------------------------------------
--!! DEBUG
weizPVP.DEBUG = false
--> Initialize Data Tables
weizPVP.Settings = {} -- Create blank global table for weizPVP Settings
weizPVP.PlayerDB = {} -- player db of detected players
weizPVP.PlayerActiveCache = {} -- Cache of players for this session

--> Nearby Count
weizPVP.NearbyCount = {}

--> Get User's Name and Realm
weizPVP.CurrentRealm = GetRealmName() -- player's realm name
weizPVP.CurrentCharName = GetUnitName("player") -- player's char name
weizPVP.PlayerLevel = UnitLevel("player") -- player's level

-------------------------------------------------------------------------------
--=> Default Settings
-------------------------------------------------------------------------------
weizPVP.Settings = {
    Enabled = true,
    --> BG/BORDER FOR ALL FRAMES
    BACKGROUNDTEXTURE = "weizPVP-Background",
    BORDERTEXTURE = "weizPVP-Border",
    BACKGROUNDCOLOR = {
        R = 0,
        G = 0,
        B = 0,
        A = 0.7
    },
    --> MAIN FRAME
    MAIN = {
        WIDTH = 260,
        HEIGHT = 0,
        BORDER_SIZE = 2
    },
    --> SCROLL FRAME
    SCROLL = {
        HEIGHT = 0,
        WIDTH = 0,
        scrollBarWidth = 4,
        SCROLLOFFET = 0
    },
    --> LIST FRAME
    LIST = {
        ROWHEIGHT = 0,
        HEIGHT = 0,
        WIDTH = 0
    },
    --> Header Settings
    HEADER = {
        HEIGHT = 34,
        WIDTH = 0,
        TEXT = {
            FONT = "Roboto Condensed BoldItalic",
            FONTSIZE = 20,
            FONTOUTLINE = "OUTLINE"
        },
        BUTTONS = {
            CONFIG = true,
            STATS = true,
            CLEAR = true,
            PREV = true,
            NEXT = true,
            CLOSE = true
        }
    },
    --> FOOTER FRAME
    FOOTER = {
        HEIGHT = 22,
        TEXT = {
            -- FONT = "Accidental Presidency",
            FONT = "Roboto Condensed Bold",
            FONTSIZE = 16,
            FONTOUTLINE = "OUTLINE"
        }
    },
    --> BARS
    BARS = {
        ALPHA_DEFAULT = 1,
        ALPHA_INACTIVE = 0.5,
        ALPHA_DEAD = 0.7,
        MAXNUMBER = 30,
        HEIGHT = 18,
        WIDTH = 0,
        SPACING = 1,
        NUM_DISPLAYED = 12,
        TEXTURE = "weizPVP.Bar",
        HIGHLIGHTTEXTURE = "weizPVP.Bar-Highlight",
        BGTEXTURE = "weizPVP.BarBackground",
        NAME_FONT_DIFF = -2,
        NAME_ALPHA_STEALTH = 0.6,
        LEVEL_FONT_DIFF = -2,
        GUILD_FONT_DIFF = -4
    },
    --> FONTS
    FONTS = {
        NARROW = "Accidental Presidency",
        NORMAL = "Roboto Condensed Bold",
        BOLD = "Roboto Condensed Bold",
        ITALIC = "Accidental Presidency Italic",
        BOLDITALIC = "Roboto Condensed BoldItalic"
    }
}

--=> Computed Settings
-------------------------------------------------------------------------------
weizPVP.Settings.LIST.ROWHEIGHT = weizPVP.Settings.BARS.HEIGHT + weizPVP.Settings.BARS.SPACING
weizPVP.Settings.SCROLL.HEIGHT = (weizPVP.Settings.LIST.ROWHEIGHT * weizPVP.Settings.BARS.NUM_DISPLAYED) + weizPVP.Settings.BARS.SPACING
weizPVP.Settings.SCROLL.WIDTH = weizPVP.Settings.MAIN.WIDTH
weizPVP.Settings.MAIN.HEIGHT = weizPVP.Settings.SCROLL.HEIGHT
weizPVP.Settings.LIST.HEIGHT = weizPVP.Settings.LIST.ROWHEIGHT * weizPVP.Settings.BARS.MAXNUMBER
weizPVP.Settings.LIST.WIDTH = weizPVP.Settings.SCROLL.WIDTH
weizPVP.Settings.BARS.WIDTH = weizPVP.Settings.LIST.WIDTH
weizPVP.Settings.HEADER.WIDTH = weizPVP.Settings.MAIN.WIDTH

-------------------------------------------------------------------------------
--=> LIB SHARED MEDIA ADDTIONS
-------------------------------------------------------------------------------
SM:Register("statusbar", "weizPVP.Bar", [[Interface\Addons\weizPVP\Textures\bar.tga]])
SM:Register("statusbar", "weizPVP.Bar-Highlight", [[Interface\Addons\weizPVP\Textures\bar-highlight.tga]])
SM:Register("background", "weizPVP-Background", [[Interface\Addons\weizPVP\Textures\bg.tga]])
SM:Register("background", "weizPVP.BarBackground", [[Interface\Addons\weizPVP\Textures\bar_bg.tga]])
SM:Register("border", "weizPVP-Border", [[Interface\Addons\weizPVP\Textures\border.tga]])
SM:Register("font", "Roboto Condensed Bold", [[Interface\Addons\weizPVP\Fonts\RobotoCondensed-Bold.ttf]])
SM:Register("font", "Roboto Condensed BoldItalic", [[Interface\Addons\weizPVP\Fonts\RobotoCondensed-BoldItalic.ttf]])
SM:Register("font", "Pepsi", [[Interface\Addons\weizPVP\Fonts\Pepsi.ttf]])
SM:Register("font", "Accidental Presidency", [[Interface\Addons\weizPVP\Fonts\Accidental-Presidency.ttf]])
SM:Register("font", "Accidental Presidency Italic", [[Interface\Addons\weizPVP\Fonts\Accidental-Presidency-Italic.ttf]])
SM:Register("font", "Open Sans Condensed", [[Interface\Addons\weizPVP\Fonts\OpenSansCondensed.ttf]])

--=> OnInitialize
-------------------------------------------------------------------------------
function weizPVP:OnInitialize()

    --> LOAD ACE3 DB
    self.db = LibStub("AceDB-3.0"):New("weizPVP_DB")

    --> Initialize DBs & Lookup Tables
    weizPVP:CreateAbilityDB()
    weizPVP:CreateSpecDB()

end

--> OnLoad
-------------------------------------------------------------------------------
function weizPVP:OnLoad()
end

--=> OnEnable
-------------------------------------------------------------------------------
function weizPVP:OnEnable()

    --> Get PlayerCache DB
    if self.db.profile.PlayerDB then
        weizPVP.PlayerDB = self.db.profile.PlayerDB
    end

    --> Setup Timers
    weizPVP:ScheduleRepeatingTimer("ManageExpirations", 1)
    weizPVP:ScheduleRepeatingTimer("SaveAddonData", 10)

    --> Setup Registered Events
    weizPVP:RegisterEvent("ZONE_CHANGED", "ZoneChangedEvent")
    weizPVP:RegisterEvent("ZONE_CHANGED_INDOORS", "ZoneChangedEvent")
    weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")
    weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
    weizPVP:RegisterEvent("UNIT_FACTION", "ZoneChangedEvent")
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeftCombatEvent")
    weizPVP:RegisterEvent("PLAYER_DEAD", "PlayerDeadEvent")
    weizPVP:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE", "ChannelNoticeEvent")
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    -- weizPVP:RegisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealth")
    weizPVP:RegisterEvent("UNIT_HEALTH", "UnitHealth")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

end

--=> OnDisable
-------------------------------------------------------------------------------
function weizPVP:OnDisable()
    weizPVP:UnregisterEvent("ZONE_CHANGED", "ZoneChangedEvent")
    weizPVP:UnregisterEvent("ZONE_CHANGED_INDOORS", "ZoneChangedEvent")
    weizPVP:UnregisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")
    weizPVP:UnregisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
    weizPVP:UnregisterEvent("UNIT_FACTION", "ZoneChangedEvent")
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeftCombatEvent")
    weizPVP:UnregisterEvent("PLAYER_DEAD", "PlayerDeadEvent")
    weizPVP:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE", "ChannelNoticeEvent")
    -- weizPVP:UnregisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealth")
    weizPVP:UnregisterEvent("UNIT_HEALTH", "UnitHealth")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")
end

--=> Clear List Data
-------------------------------------------------------------------------------
function weizPVP:ClearListData()
    table_wipe(weizPVP.PlayerActiveCache)
    table_wipe(weizPVP.ActiveList)
    table_wipe(weizPVP.InactiveList)
    table_wipe(weizPVP.NearbyList)
    table_wipe(weizPVP.CurrentList)
    weizPVP:RefreshCurrentList()
    weizPVP:UpdateNearbyCount()
end

--=> Save Addon Data
-------------------------------------------------------------------------------
function weizPVP:SaveAddonData()
    self.db.profile.Settings = weizPVP.Settings
    self.db.profile.PlayerDB = weizPVP.PlayerDB
end
